﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.WinControls.RadControlSpy;
using Telerik.WinControls.UI;

namespace PropertyGridColumnWidth
{
    public partial class Form1 : Form
    {
        MyPropertyGrid prop;
        public Form1()
        {
            InitializeComponent();

            this.prop = new MyPropertyGrid
            {
                Parent = this,
                Dock = DockStyle.Fill
            };

            prop.SelectedObject = this;

            new RadButton
            {
                Parent = this,
                Dock = DockStyle.Bottom
            }.Click += (s, e) =>
                {
                };

            this.Load += Form1_Load;

            new RadControlSpyForm().Show();
        }

        void Form1_Load(object sender, EventArgs e)
        {
            prop.PropertyGridElement.PropertyTableElement.ValueColumnWidth = 50;
            var textBox = prop.PropertyGridElement.SplitElement.HelpElement.FindDescendant<RadTextBoxControlElement>();
            textBox.BackColor = Color.FromArgb(212, 230, 252);
            textBox.DrawBorder = false;
        }
    }
    public class MyPropertyGrid : RadPropertyGrid
    {
        protected override PropertyGridElement CreatePropertyGridElement()
        {
            return new MyPropertyGridElement();
        }
    }

    public class MyPropertyGridElement : PropertyGridElement
    {
        protected override PropertyGridSplitElement CreateSplitElement()
        {
            return new MyPropertyGridSplitElement();
        }

        protected override Type ThemeEffectiveType
        {
            get
            {
                return typeof(PropertyGridElement);
            }
        }
    }

    public class MyPropertyGridSplitElement : PropertyGridSplitElement
    {
        protected override PropertyGridHelpElement CreateHelpElement()
        {
            return new MyPropertyGridHelpElement();
        }
    }

    public class MyPropertyGridHelpElement : PropertyGridHelpElement
    {
        private RadScrollViewer scrollViewer;
        private Telerik.WinControls.Layouts.DockLayoutPanel panel;
        private PropertyGridHelpTitleElement titleElement;
        private PropertyGridHelpContentElement contentElement;
        private RadTextBoxControlElement textBox;

        protected override void CreateChildElements()
        {
            this.scrollViewer = new RadScrollViewer();
            this.scrollViewer.ShowFill = this.scrollViewer.ShowBorder = false;

            this.panel = new Telerik.WinControls.Layouts.DockLayoutPanel();
            panel.LastChildFill = true;

            this.titleElement = new PropertyGridHelpTitleElement();
            this.contentElement = new PropertyGridHelpContentElement();
            this.contentElement.TextWrap = true;
            this.panel.Children.Add(this.titleElement);
            Telerik.WinControls.Layouts.DockLayoutPanel.SetDock(this.titleElement, Telerik.WinControls.Layouts.Dock.Top);
            this.panel.Children.Add(this.contentElement);

            typeof(PropertyGridHelpElement).GetField("titleElement", BindingFlags.Instance | BindingFlags.NonPublic).SetValue(this, this.titleElement);
            typeof(PropertyGridHelpElement).GetField("contentElement", BindingFlags.Instance | BindingFlags.NonPublic).SetValue(this, this.contentElement);

            this.contentElement.Visibility = Telerik.WinControls.ElementVisibility.Collapsed;
            this.textBox = new RadTextBoxControlElement();
            this.contentElement.RadPropertyChanged += contentElement_RadPropertyChanged;
            this.panel.Children.Add(this.textBox);
            this.textBox.IsReadOnly = true;
            this.textBox.IsReadOnlyCaretVisible = false;
            this.textBox.TextWrap = true;
            this.textBox.WordWrap = true;
            this.textBox.Multiline = true;

            this.scrollViewer.Viewport = this.panel;
            this.Children.Add(this.scrollViewer);
        }

        void contentElement_RadPropertyChanged(object sender, Telerik.WinControls.RadPropertyChangedEventArgs e)
        {
            if (e.Property == PropertyGridHelpContentElement.TextProperty)
            {
                this.textBox.Text = this.contentElement.Text;
            }
        }

        protected override Type ThemeEffectiveType
        {
            get
            {
                return typeof(PropertyGridHelpElement);
            }
        }

        protected override SizeF ArrangeOverride(SizeF finalSize)
        {
            SizeF baseArrange = base.ArrangeOverride(finalSize);
            this.scrollViewer.Arrange(new RectangleF(PointF.Empty, finalSize));
            this.contentElement.MaxSize = new Size((int)finalSize.Width - 17, 0);
            this.textBox.MaxSize = this.contentElement.MaxSize;
            return baseArrange;
        }
    }
}
